@@ -61,11 +61,6 @@ ERROR, INFO, DEBUG = range(3)
 ## Names changed for clarity: mod1->alt, mod4->winkey, mod5->altgr 
 MODIFIERS = "shift", "lock", "control", "alt", "mod2", "mod3", "winkey", "altgr"
 
-########################################
-## From Peter Norvig's Infrequently Answered Questions
-def create_dict(**dict): 
-	return dict
-
 ####################################################
 def debug(text, verbose_level, level, exit=None):
 	"""Give some debug messages.
@@ -87,6 +82,32 @@ def run_command(command):
 	shell = os.getenv("SHELL", "/bin/sh")
 	os.spawnvp(os.P_NOWAIT, shell, [os.path.basename(shell), "-c", command])
 
+################################
+################################
+class FileChooserDialog(gtk.FileChooserDialog):
+	"""Gtk file chooser Dialog with facilities for buttons. Also, Escape key closes window"""
+	################################
+	def __init__(self, *args, **kwargs):
+		gtk.FileChooserDialog.__init__(self,*args, **kwargs)
+		self.connect("key-press-event", self.on_fcd_key_press_event)
+
+	################################
+	def on_fcd_key_press_event(self, widget, event):
+		if event.keyval == gtk.keysyms.Escape:
+			self.emit("response", gtk.RESPONSE_REJECT)
+
+	################################
+	def add_filterdef(self, filterdef):
+		"""Filter Definition: list of (strFilterName, listMimeTypes, listPatterns)"""
+		for name, mime_types, patterns in filterdef:
+			filter = gtk.FileFilter()
+			filter.set_name(name)
+			for mt in mime_types:
+				filter.add_mime_type(mt)
+			for pat in patterns:
+				filter.add_pattern(pat)
+			self.add_filter(filter)
+
 ######################################
 ######################################
 class XhotkeysServer:
@@ -187,7 +208,7 @@ class XhotkeysServer:
 					hktype, keycode = self.string_to_keycode(key)
 				except: 
 					self.debug("error parsing hotkey: %s (entry %s ignored)" %(key, comb["name"]), ERROR); continue
-			newcomb = create_dict(name=comb["name"], command=comb["command"], \
+			newcomb = dict(name=comb["name"], command=comb["command"], \
 				hktype=hktype, mask=mask, keycode=keycode)
 			self.combinations.append(newcomb)
 		self.configure()
@@ -352,7 +373,7 @@ class Xhotkeys:
 				name, options = [x.strip() for x in line.split("=", 1)]
 				hotkey, command = options.split(":", 1)
 			except: continue
-			comb = create_dict(name=name, hotkey=hotkey, command=command)
+			comb = dict(name=name, hotkey=hotkey, command=command)
 			for combination in self.combinations:
 				if combination["name"] == name: 
 					self.debug("hotkey name (%s) already loaded, ignored" %name, ERROR)
@@ -411,12 +432,6 @@ class Xhotkeys:
 		self.debug("%s combinations written" %len(self.combinations), INFO)
 		fd.close()
 
-	#######################################
-	def filechooser_file(self, widget):
-		command = widget.get_filename()
-		self.widgets["command"].set_text(command)
-		widget.destroy()
-
 	#######################################
 	def set_hotkey_text(self, text):
 		self.widgets["hotkey"].set_text(text.title())
@@ -448,7 +463,7 @@ class Xhotkeys:
 		if not keycodes: self.set_hotkey_text(self.current_comb["hotkey"]); return			
 		mask = self.keycodes_to_mask(keycodes)
 		keycode = keycodes[-1]
-		newcomb = create_dict(mask=mask, keycode=keycode, hotkey=hotkey)
+		newcomb = dict(mask=mask, keycode=keycode, hotkey=hotkey)
 
 		for index, comb in enumerate(self.combinations):
 			if comb == self.old_comb: continue
@@ -523,7 +538,7 @@ class Xhotkeys:
 	def get_combinations(self, row):
 		if row < 0: return
 		cr = self.combinations[row]
-		return create_dict(name=cr["name"], command=cr["command"], hotkey=cr["hotkey"])
+		return dict(name=cr["name"], command=cr["command"], hotkey=cr["hotkey"])
 
 	#######################################		
 	def apply_widgets(self, function, **args):
@@ -624,7 +639,7 @@ class Xhotkeys:
 		names.sort()
 		name = names[-1][1]
 		hotkey = self.keycodes_to_string([])
-		self.current_comb = create_dict(name=name, command=command, hotkey=hotkey)
+		self.current_comb = dict(name=name, command=command, hotkey=hotkey)
 		self.open_add_window(position=dest_row)
 		self.old_comb = None
 		self.textbox_check(self.widgets["name"], "name")
@@ -676,7 +691,7 @@ class Xhotkeys:
 	#######################################
 	def on_add_clicked(self, widget):
 		hotkey = self.keycodes_to_string([])
-		self.current_comb = create_dict(name="", command="", hotkey=hotkey)
+		self.current_comb = dict(name="", command="", hotkey=hotkey)
 		self.apply_widgets("set_sensitive", accept=False, test=False)		
 		self.open_add_window()
 		self.old_comb = None
@@ -756,9 +771,14 @@ class Xhotkeys:
 
 	#######################################
 	def on_browse_clicked(self, widget):
-		self.widgets["filechooser"] = gtk.FileChooserDialog()
-		self.widgets["filechooser"].connect("file-activated", self.filechooser_file)
-		self.widgets["filechooser"].run()
+		buttons = (gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT, \
+			gtk.STOCK_OPEN, gtk.RESPONSE_ACCEPT)
+		fchooser = FileChooserDialog(buttons=buttons, action=gtk.FILE_CHOOSER_ACTION_OPEN)
+		if fchooser.run() == gtk.RESPONSE_ACCEPT:
+			command = fchooser.get_filename()
+			if os.access(command, os.X_OK):
+				self.widgets["command"].set_text(command)
+		fchooser.destroy()
 
 	#########################################
 	def on_name_changed(self, widget):
